<?php

namespace forge12\woocommerce\free\accessories {
    if (!defined('ABSPATH')) {
        exit;
    }

    /**
     * Class OptionalAccessoriesToWooCommerce
     * Responsible to handle the admin settings for the accessoire field
     *
     * @package forge12\woocommerce\free\accessories
     */
    class OptionalAccessoriesToWooCommerce
    {
        /**
         * @var OptionalAccessoriesToWooCommerce
         */
        private static $instance = null;

        /**
         * @return OptionalAccessoriesToWooCommerce|null
         */
        public static function getInstance()
        {
            if (self::$instance == null) {
                self::$instance = new self;
            }
            return self::$instance;
        }

        /**
         * Admin constructor.
         */
        public function __construct()
        {
            add_action('admin_init', array($this, '_addHooks'));
            add_action('admin_enqueue_scripts', array($this, '_addStyles'));
        }

        /**
         * Add the styles for the form
         */
        public function _addStyles($hook)
        {
            wp_enqueue_style('f12-woo-accessories-admin', plugins_url('admin-style.css', __FILE__));
        }

        /**
         * Add the hooks responsible to add the accessoire option field to the products
         * and to store the informations.
         */
        public function _addHooks()
        {
            $Settings = UIOptionalAccessories::getSettings();

            // Check if the settings are enabled
            if ($Settings['enable'] == 1) {
                add_action('woocommerce_product_options_related', array($this, '_renderOptionalAccessories'));
                add_action('woocommerce_process_product_meta', array($this, '_save'));
            }
        }

        /**
         * Save the accessoire ids for the given product.
         *
         * @param int $post_id The ID of the current product
         */
        public function _save($post_id)
        {
            $product = wc_get_product($post_id);

            if (null == $product || !$product) {
                return;
            }

            $accessoryIds = isset($_POST['accessory_ids']) ? array_map('absint', $_POST['accessory_ids']) : array();
            $accessoryTitle = isset($_POST['accessory_title']) ? sanitize_text_field($_POST['accessory_title']) : '';
            $accessoryDescription = isset($_POST['accessory_description']) ? sanitize_textarea_field($_POST['accessory_description']) : '';

            $product->update_meta_data('f12-woo-accessoire_ids', implode(",", $accessoryIds));
            $product->update_meta_data('f12-woo-accessoire_title', $accessoryTitle);
            $product->update_meta_data('f12-woo-accessoire_description', $accessoryDescription);
            $product->save();
        }

        /**
         * Return a comma separated string containing all products already defined within the category.
         * @param $post_id
         * @return string
         */
        private function getAccessoriesGlobalyDefined($post_id)
        {
            $result = array();
            $product = wc_get_product($post_id);

            if (null == $product || !$product) {
                return implode('', $result);
            }

            $arr_category_ids = $product->get_category_ids();
            $arr_ids = array();

            foreach ($arr_category_ids as $key => $term_id) {
                if (!isset($arr_ids[$term_id])) {
                    $arr_ids[$term_id] = array();
                }

                $arr_ids[$term_id] = array_merge($arr_ids, accessories::getAccessoireIDsByTermID($term_id, 'optional'));
            }

            foreach ($arr_ids as $term_id => $arr_product_ids) {
                foreach ($arr_product_ids as $product_id) {
                    $product = wc_get_product($product_id);
                    if (false !== $product) {
                        $result[] = '<a href="' . esc_url(admin_url('term.php?taxonomy=product_cat&tag_ID=' . $term_id)) . '" target="_blank">' . esc_html($product->get_title()) . '</a>';
                    }
                }
            }

            return implode(', ', $result);
        }

        /**
         * Get the defined Title for the Post
         * @param $post_id
         * @return string
         */
        public static function getTitle($post_id)
        {
            $product = wc_get_product($post_id);

            if (!$product) {
                return '';
            }

            return $product->get_meta('f12-woo-accessoire_title', true);
        }

        /**
         * Get the description
         * @param $post_id
         * @return string
         */
        public static function getDescription($post_id)
        {
            $product = wc_get_product($post_id);

            if (!$product) {
                return '';
            }

            return $product->get_meta('f12-woo-accessoire_description', true);
        }

        /**
         * Render the input field for the accessories on the product page.
         */
        public function _renderOptionalAccessories()
        {
            global $post;

            $settings = \forge12\woocommerce\free\accessories\UIOptionalAccessories::getSettings();
            $settings_title = $settings['title'];
            $settings_description = $settings['description'];

            $accessories_title = self::getTitle($post->ID);
            $accessories_description = self::getDescription($post->ID);
            $accessories_ids = accessories::getAccessoireIDsByProductID($post->ID, 'optional');

            ?>
            <div class="options_group">
                <p class="form-field">
                    <label>
                        <strong>
                            <?php _e('Optional Accessories', 'f12-woo-accessories'); ?>
                        </strong>
                    </label>
                </p>
                <p class="form-field">
                    <label for="accessory_title"><?php esc_html_e('Title', 'f12-woo-accessories'); ?></label>
                    <input type="text" id="accessory_title" name="accessory_title"
                           value="<?php esc_attr_e($accessories_title); ?>"/>
                </p>
                <?php if (!empty($settings_title)): ?>
                    <p class="form-field">
                        <label>
                            <?php _e('Defined by Settings: ', 'f12-woo-accessories'); ?>
                        </label>
                        <span><?php esc_html_e($settings_title); ?></span>
                    </p>
                <?php endif; ?>
                <p class="form-field">
                    <label for="accessory_description"><?php esc_html_e('Description', 'f12-woo-accessories'); ?></label>
                    <textarea id="accessory_description"
                              name="accessory_description"><?php echo esc_textarea($accessories_description); ?></textarea>
                </p>
                <?php if (!empty($settings_description)): ?>
                    <p class="form-field">
                        <label>
                            <?php _e('Defined by Settings: ', 'f12-woo-accessories'); ?>
                        </label>
                        <span><?php esc_html_e($settings_description); ?></span>
                    </p>
                <?php endif; ?>
                <p class="form-field">
                    <label for="accessory_ids"><?php esc_html_e('Optional Accessories', 'f12-woo-accessories'); ?></label>
                    <select class="wc-product-search" multiple="multiple" style="width: 50%;" id="accessory_ids"
                            name="accessory_ids[]"
                            data-placeholder="<?php esc_attr_e('Search for a product&hellip;', 'woocommerce'); ?>"
                            data-action="woocommerce_json_search_products_and_variations"
                            data-exclude="<?php echo intval($post->ID); ?>">
                        <?php
                        foreach ($accessories_ids as $product_id) {
                            $product = wc_get_product($product_id);

                            if (null != $product && false != $product) {
                                echo '<option value="' . esc_attr($product_id) . '"' . selected(true, true, false) . '>' . esc_html(wp_strip_all_tags($product->get_formatted_name())) . '</option>';
                            }
                        }
                        ?>
                    </select> <?php echo wc_help_tip(__('Accessories are products that are used with the current product, for example, products that are required to maintain the product.', 'f12-woo-accessories')); // WPCS: XSS ok.
                    ?>
                </p>
                <?php
                $definedByCategory = $this->getAccessoriesGlobalyDefined($post->ID);
                if (!empty($definedByCategory)):
                    ?>
                    <p class="form-field">
                        <label>
                            <?php _e('Defined by Category: ', 'f12-woo-accessories'); ?>
                        </label>
                        <span><?php echo $definedByCategory; ?></span>
                    </p>
                <?php endif; ?>
            </div>
            <?php
        }
    }
}